@@ -58,7 +58,7 @@ class CinnamonFavoriteApplet extends Applet.IconApplet {
 
         this.favoritesBox = new St.BoxLayout({ vertical:true });
         this.favoritesScrollBox.add_actor(this.favoritesBox);
-        this.favoritesScrollBox.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC);
+        this.favoritesScrollBox.set_policy(St.PolicyType.NEVER, St.PolicyType.AUTOMATIC);
         this.favoritesScrollBox.add_style_class_name("vfade");
 
         this.settings = new Settings.AppletSettings(this, metadata.uuid, this.instance_id);
@@ -72,7 +72,7 @@ class CinnamonFavoriteApplet extends Applet.IconApplet {
 
         this.favorites_id = this.favorites.connect('changed', ()=>this._refreshFavorites());
 
-        global.settings.connect('changed::panel-edit-mode', ()=>this._on_panel_edit_mode_changed());
+        this.edit_mode_id = global.settings.connect('changed::panel-edit-mode', () => this._on_panel_edit_mode_changed());
     }
 
     settings_changed() {
@@ -90,6 +90,11 @@ class CinnamonFavoriteApplet extends Applet.IconApplet {
             this.favorites_id = 0;
         }
 
+        if (this.edit_mode_id > 0) {
+            global.settings.disconnect(this.edit_mode_id);
+            this.edit_mode_id = 0;
+        }
+
         this.favorites = null;
     }
 
