@@ -31,8 +31,9 @@ ROW_SIZE = 32
 
 UNSAFE_ITEMS = ['spawn_sync', 'spawn_command_line_sync', 'GTop', 'get_file_contents_utf8_sync']
 
-LANGUAGE_CODE = "C"
+LANGUAGE_CODE = LONG_LANGUAGE_CODE = "C"
 try:
+    LONG_LANGUAGE_CODE = locale.getlocale()[0]
     LANGUAGE_CODE = locale.getlocale()[0].split("_")[0]
 except:
     pass
@@ -685,6 +686,7 @@ class DownloadSpicesRow(Gtk.ListBoxRow):
         self.description = data['description']
         self.score = data['score']
         self.timestamp = data['last_edited']
+        self.subject = data['last_commit_subject']
 
         self.author = ""
         if 'author_user' in data:
@@ -692,12 +694,15 @@ class DownloadSpicesRow(Gtk.ListBoxRow):
                 self.author = data['author_user']
 
         if 'translations' in data.keys():
-            key = f'name_{LANGUAGE_CODE}'
-            if key in data['translations'].keys():
-                self.name = data['translations'][key]
-            key = f'description_{LANGUAGE_CODE}'
-            if key in data['translations'].keys():
-                self.description = data['translations'][key]
+            for key in (f'name_{LONG_LANGUAGE_CODE}', f'name_{LANGUAGE_CODE}'):
+                if key in data['translations'].keys():
+                    self.name = data['translations'][key]
+                    break
+
+            for key in (f'description_{LONG_LANGUAGE_CODE}', f'description_{LANGUAGE_CODE}'):
+                if key in data['translations'].keys():
+                    self.description = data['translations'][key]
+                    break
 
         self.has_update = False
 
@@ -749,7 +754,11 @@ class DownloadSpicesRow(Gtk.ListBoxRow):
 
         description_label = SettingsLabel()
         description_markup = GLib.markup_escape_text(sanitize_html(self.description))
-        description_label.set_markup(f'<small>{description_markup}</small>')
+        if self.spices.get_has_update(uuid):
+            subject_markup = GLib.markup_escape_text(sanitize_html(self.subject))
+            description_label.set_markup(f'<small>{description_markup}</small>\n<small><i>{subject_markup}</i></small>')
+        else:
+            description_label.set_markup(f'<small>{description_markup}</small>')
         description_label.set_margin_top(2)
         desc_box.pack_start(description_label, False, False, 0)
 
