@@ -7,7 +7,6 @@ const Signals = imports.signals;
 const Lang = imports.lang;
 const St = imports.gi.St;
 const Cinnamon = imports.gi.Cinnamon;
-const Gdk = imports.gi.Gdk;
 
 const Main = imports.ui.main;
 const MessageTray = imports.ui.messageTray;
@@ -137,7 +136,7 @@ Overview.prototype = {
                 let dt = (event.get_time() - this._lastMotionTime) / 1000;
                 let passedHalf = Math.abs(distance / difference) > 0.5;
 
-                /* Switch to the next page if the scroll ammount is more
+                /* Switch to the next page if the scroll amount is more
                    than half the page width or is faster than 25px/s.
                    This number comes from experimental tests. */
                 if (Math.abs(distance) > dt * 25 || passedHalf) {
@@ -255,7 +254,13 @@ Overview.prototype = {
         this._background.set_position(0, 0);
         this._group.add_actor(this._background);
 
-        let desktopBackground = Meta.X11BackgroundActor.new_for_display(global.display);
+        let desktopBackground;
+        if (!Meta.is_wayland_compositor()) {
+            desktopBackground = Meta.X11BackgroundActor.new_for_display(global.display);
+        } else {
+            desktopBackground = new Clutter.Actor();
+        }
+
         this._background.add_actor(desktopBackground);
 
         let backgroundShade = new St.Bin({style_class: 'workspace-overview-background-shade'});
@@ -276,7 +281,7 @@ Overview.prototype = {
         this._coverPane.hide();
 
         // Disable unredirection while in the overview
-        Meta.disable_unredirect_for_screen(global.screen);
+        Meta.disable_unredirect_for_display(global.display);
         this._group.show();
 
         this.workspacesView = new WorkspacesView.WorkspacesView();
@@ -289,7 +294,7 @@ Overview.prototype = {
             Tweener.addTween(this._group, {
                 opacity: 255,
                 transition: 'easeOutQuad',
-                time: ANIMATION_TIME,
+                time: ANIMATION_TIME * 0.45,
                 onComplete: this._showDone,
                 onCompleteScope: this
             });
@@ -407,7 +412,7 @@ Overview.prototype = {
             // Make other elements fade out.
             Tweener.addTween(this._group, {
                 opacity: 0,
-                transition: 'easeOutQuad',
+                transition: 'easeInQuad',
                 time: ANIMATION_TIME,
                 onComplete: this._hideDone,
                 onCompleteScope: this
